/**************************************************** */
/* Rule Set Based Access Control                      */
/* Implementation of the Access Control Decision      */
/* Facility (ADF) - Security Information Modification */
/* File: rsbac/adf/sim/main.c                         */
/*                                                    */
/* Author and (c) 1999: Amon Ott                      */
/*                                                    */
/* Last modified: 13/May/99                           */
/**************************************************** */

#include <linux/string.h>
#include <rsbac/aci.h>
#include <rsbac/adf_main.h>
#include <rsbac/error.h>
#include <rsbac/helpers.h>
#include <rsbac/getname.h>

#ifndef NULL
#define NULL ((void *) 0)
#endif

/************************************************* */
/*           Global Variables                      */
/************************************************* */

/************************************************* */
/*          Internal Help functions                */
/************************************************* */

static enum rsbac_adf_req_ret_t
         check_role_sim(  enum  rsbac_target_t          target,
                          union rsbac_target_id_t       tid,
                                rsbac_uid_t             owner)
  {
    union rsbac_target_id_t       i_tid;
    union rsbac_attribute_value_t i_attr_val1;
    union rsbac_attribute_value_t i_attr_val2;
    boolean inherit;

    /* test target's data_type */
    switch(target)
      {
        case T_FILE:
        case T_DIR:
          inherit = TRUE;
          break;
        default:
          inherit = FALSE;
      }
    if (rsbac_get_attr(target,
                       tid,
                       A_data_type,
                       &i_attr_val1,
                       inherit))
      {
        printk(KERN_WARNING "check_role_sim(): rsbac_get_attr() returned error!\n");
        return(NOT_GRANTED);
      }
    /* Access to other data_types than SI is always granted */
    if (i_attr_val1.data_type != DT_SI)
      return(GRANTED);

    /* test owner's system_role */
    i_tid.user = owner;
    if (rsbac_get_attr(T_USER,
                       i_tid,
                       A_system_role,
                       &i_attr_val2,
                       TRUE))
      {
        printk(KERN_WARNING "check_role_sim(): rsbac_get_attr() returned error!\n");
        return(NOT_GRANTED);
      }
      
    /* Access is granted, if owner's system_role is security_officer */

    if (i_attr_val2.system_role == SR_security_officer)
      return(GRANTED);
    else
      return(NOT_GRANTED);
  }

/************************************************* */
/*          Externally visible functions           */
/************************************************* */

enum rsbac_adf_req_ret_t
   rsbac_adf_request_sim (enum  rsbac_adf_request_t     request,
                                rsbac_pid_t             caller_pid,
                          enum  rsbac_target_t          target,
                          union rsbac_target_id_t       tid,
                          enum  rsbac_attribute_t       attr,
                          union rsbac_attribute_value_t attr_val,
                                rsbac_uid_t             owner)
  {
    enum  rsbac_adf_req_ret_t result = DO_NOT_CARE;
    union rsbac_target_id_t       i_tid;
    union rsbac_attribute_value_t i_attr_val1;
#ifdef CONFIG_RSBAC_SIM_ROLE_PROT
    union rsbac_attribute_value_t i_attr_val2;
#endif
    int err=0;

    switch (request)
      {
        case R_ADD_TO_KERNEL:
            switch(target)
              {
                case T_FILE:
                case T_NONE:
                  return(DO_NOT_CARE);

                /* all other cases are undefined */
                default:
                  return(DO_NOT_CARE);
              }


        case R_ALTER:
            /* only for IPC */
            switch(target)
              {
                case T_IPC:
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
             }

        case R_APPEND_OPEN:
        case R_READ_WRITE_OPEN:
            switch(target)
              {
                case T_FILE:
                case T_DEV:
                case T_IPC:
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_CHANGE_GROUP:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_IPC:
                  return(check_role_sim(target,tid,owner));

                case T_PROCESS:
                case T_USER:
                  return(DO_NOT_CARE);
                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_CHANGE_OWNER:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_IPC:
                  return(check_role_sim(target,tid,owner));
                  
                case T_PROCESS:
                  #ifdef CONFIG_RSBAC_SIM_ROLE_PROT
                  if(attr != A_owner)
                    return(UNDEFINED);
                  /* Administrator or secoff? */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* if general user or secoff, then grant */
                  if (   (i_attr_val1.system_role == SR_user)
                      || (i_attr_val1.system_role == SR_security_officer)
                     )
                    return(GRANTED);
                  /* old owner is sys-admin */
                  /* get target user's role */
                  i_tid.user = attr_val.owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val2,
                                     TRUE))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* if target is security officer -> deny */
                  if(i_attr_val2.system_role == SR_security_officer)
                    return(NOT_GRANTED);
                  else
                    return(GRANTED);
                  #endif /* ROLE_PROT */

                  /* fall through */
                case T_NONE:
                  return(DO_NOT_CARE);
                /* all other cases are undefined */
                default:
                  return(DO_NOT_CARE);
              }

        case R_CHDIR:
        case R_READ:
        case R_SEARCH:
            switch(target)
              {
                case T_DIR: 
                  return(DO_NOT_CARE);

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        /* Creating dir or (pseudo) file IN target dir! */
        case R_CREATE:
            switch(target)
              {
                case T_DIR: 
                  return(check_role_sim(target,tid,owner));
                /* IPC is always granted */
                case T_IPC:
                  return(GRANTED);

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_DELETE:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_IPC:
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_EXECUTE:
            switch(target)
              {
                case T_FILE:
                  return(DO_NOT_CARE);

                /* all other cases are undefined */
                default:
                  return(DO_NOT_CARE);
              }

        case R_GET_PERMISSIONS_DATA:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_IPC:
                case T_SCD:
                  return(DO_NOT_CARE);
                default:
                  return(DO_NOT_CARE);
               };

        case R_GET_STATUS_DATA:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_IPC:
                  return(DO_NOT_CARE);
                case T_SCD:
                  /* target rsbaclog? only for secoff */
                  if (tid.scd != ST_rsbaclog)
                    return(GRANTED);
                  /* Secoff? */
                  i_tid.user = owner;
                  if ((err=rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE)))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_request_sim(): rsbac_get_attr() returned error %i!\n",err);
                      return(NOT_GRANTED);
                    }
                  /* grant only for secoff */
                  if (i_attr_val1.system_role == SR_security_officer)
                    return(GRANTED);
                  else
                    return(NOT_GRANTED);
                default:
                  return(DO_NOT_CARE);
               };

        case R_LINK_HARD:
        case R_TRUNCATE:
            switch(target)
              {
                case T_FILE:
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_MODIFY_ACCESS_DATA:
        case R_RENAME:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_MODIFY_ATTRIBUTE:
            switch(attr)
              { /* owner must be changed by other request to prevent inconsistency */
                case A_owner:
                  return(NOT_GRANTED);
                case A_system_role:
                case A_data_type:
                case A_log_array_low:
                case A_log_array_high:
                #ifdef CONFIG_RSBAC_SIM_AUTH_PROT
                case A_auth_may_setuid:
                case A_auth_may_set_cap:
                case A_auth_add_f_cap:
                case A_auth_remove_f_cap:
                #endif
                /* All attributes (remove target!) */
                case A_none:
                  /* Security Officer? */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* if sec_officer, then grant */
                  if (i_attr_val1.system_role == SR_security_officer)
                    return(GRANTED);
                  else
                    return(NOT_GRANTED);

                default:
                  return(DO_NOT_CARE);
              }

        case R_MODIFY_PERMISSIONS_DATA:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_IPC:
                  return(check_role_sim(target,tid,owner));
                  
                case T_SCD:
                  /* Security Officer? */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* if sec_officer, then grant */
                  if (i_attr_val1.system_role == SR_security_officer)
                    return(GRANTED);
                  /* For booting: if administrator and ioports, then grant */
                  if (   (i_attr_val1.system_role == SR_administrator)
                      && (tid.scd == ST_ioports) )
                    return(GRANTED);
                  else
                    return(NOT_GRANTED);
                  
                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_MODIFY_SYSTEM_DATA:
            switch(target)
              {
                case T_SCD:
                  /* target not rsbaclog? no problem -> grant */
                  if (tid.scd != ST_rsbaclog)
                    return(GRANTED);
                  /* Get role */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* grant only for secoff */
                  if (i_attr_val1.system_role == SR_security_officer)
                    return(GRANTED);
                  else
                    return(NOT_GRANTED);
                  
                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_MOUNT:
        case R_UMOUNT:
        case R_WRITE:
            switch(target)
              {
                case T_DIR:
                case T_DEV:
                  /* test access to dir/dev */
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_READ_ATTRIBUTE:
                  return(DO_NOT_CARE);

        case R_READ_OPEN:
            switch(target)
              {
                case T_FILE:
                case T_DIR:
                case T_DEV:
                case T_IPC:
                  return(DO_NOT_CARE);

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_REMOVE_FROM_KERNEL:
        case R_SHUTDOWN:
            switch(target)
              {
                case T_NONE:
                  return(DO_NOT_CARE);

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_SWITCH_LOG:
            switch(target)
              {
                case T_NONE:
                  /* test owner's system_role */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* security officer? -> grant  */
                  if (i_attr_val1.system_role == SR_security_officer)
                    return(GRANTED);
                  else
                    return(NOT_GRANTED);

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_SWITCH_MODULE:
            switch(target)
              {
                case T_NONE:
                  /* we need the switch_target */
                  if(attr != A_switch_target)
                    return(UNDEFINED);
                  /* do not care for other modules */
                  if(   (attr_val.switch_target != SIM)
                     #ifdef CONFIG_RSBAC_SIM_AUTH_PROT
                     && (attr_val.switch_target != AUTH)
                     #endif
                    )
                    return(DO_NOT_CARE);
                  /* test owner's system_role */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING "rsbac_adf_request_sim(): rsbac_get_attr() returned error!\n");
                      return(NOT_GRANTED);
                    }
                  /* security officer? -> grant  */
                  if (i_attr_val1.system_role == SR_security_officer)
                    return(GRANTED);
                  else
                    return(NOT_GRANTED);

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }

        case R_WRITE_OPEN:
            switch(target)
              {
                case T_DEV:
                case T_FILE:
                case T_IPC:
                  return(check_role_sim(target,tid,owner));

                /* all other cases are undefined */
                default: return(DO_NOT_CARE);
              }


/*********************/
        default: return DO_NOT_CARE;
      }

    return(result);
  }; /* end of rsbac_adf_request_sim() */


/*****************************************************************************/
/* If the request returned granted and the operation is performed,           */
/* the following function can be called by the AEF to get all aci set        */
/* correctly. For write accesses that are performed fully within the kernel, */
/* this is usually not done to prevent extra calls, including R_CLOSE for    */
/* cleaning up.                                                              */
/* The second instance of target specification is the new target, if one has */
/* been created, otherwise its values are ignored.                           */
/* On success, 0 is returned, and an error from rsbac/error.h otherwise.     */

int  rsbac_adf_set_attr_sim(
                      enum  rsbac_adf_request_t     request,
                            rsbac_pid_t             caller_pid,
                      enum  rsbac_target_t          target,
                      union rsbac_target_id_t       tid,
                      enum  rsbac_target_t          new_target,
                      union rsbac_target_id_t       new_tid,
                      enum  rsbac_attribute_t       attr,
                      union rsbac_attribute_value_t attr_val,
                            rsbac_uid_t             owner)
  {
    union rsbac_target_id_t       i_tid;
    union rsbac_attribute_value_t i_attr_val1;
    union rsbac_attribute_value_t i_attr_val2;

    switch (request)
      {
        case R_APPEND_OPEN:
        case R_READ_WRITE_OPEN:
            switch(target)
              {
                case T_FILE:
                case T_DEV:
                case T_IPC:
                  return(0);

                /* all other cases are undefined */
                default:
                  return(0);
              }

        case R_CHANGE_OWNER:
            switch (target)
              {
                case T_PROCESS:
                  /* setting owner for process is done in main dispatcher */
                  return(0);
                    
                /* We do not care about those cases here */
                case T_FILE:
                case T_DIR:
                case T_IPC:
                case T_NONE:
                  return(0);
                /* all other cases are undefined */
                default:
                  return(0);
              }

        case R_CHDIR:
        case R_MOUNT:
        case R_READ:
        case R_SEARCH:
            switch(target)
              {
                case T_DIR:
                  return(0);

                /* all other cases are undefined */
                default:
                  return(0);
              }

        case R_CLONE:
            if (target == T_PROCESS)
              {
#ifdef CONFIG_RSBAC_USE_RSBAC_OWNER
                  /* Get owner from first process (provided on call) */
                  i_attr_val1.owner = owner;
                  /* Set owner for new process */
                  if (rsbac_set_attr(T_PROCESS,
                                     new_tid,
                                     A_owner,
                                     i_attr_val1))
                    {
                      printk(KERN_WARNING "rsbac_adf_set_attr_sim(): rsbac_set_attr() returned error!\n");
                      return(-RSBAC_EWRITEFAILED);
                    }
#endif
                return(0);
              }
            else
              return(0);


        case R_CREATE:
            switch(target)
              {
                /* Creating dir or (pseudo) file IN target dir! */
                case T_DIR:
                  /* Mode of created item is ignored! */
                  /* Inherit data_type from target DIR - but only, if not */
                  /* inherited by default anyway. */
                  #ifdef CONFIG_RSBAC_DEF_INHERIT
                    i_attr_val1.data_type = DT_inherit;
                  #else
                    /* get target data_type */
                    if (rsbac_get_attr(T_DIR,
                                       tid,
                                       A_data_type,
                                       &i_attr_val1,
                                       TRUE))
                      {
                        printk(KERN_WARNING
                               "rsbac_adf_set_attr_sim(): rsbac_get_attr() returned error!\n");
                        return(NOT_GRANTED);
                      }
                  #endif
                  /* And set it for new item */
                  if (rsbac_set_attr(new_target,
                                     new_tid,
                                     A_data_type,
                                     i_attr_val1) )
                    {
                      printk(KERN_WARNING "rsbac_adf_set_attr_sim(): rsbac_set_attr() returned error!\n");
                      return(-RSBAC_EWRITEFAILED);
                    }
                  return(0);

                case T_IPC: 
                  /* Get system_role of process owner... */
                  i_tid.user = owner;
                  if (rsbac_get_attr(T_USER,
                                     i_tid,
                                     A_system_role,
                                     &i_attr_val1,
                                     TRUE))
                    {
                      printk(KERN_WARNING
                             "rsbac_adf_set_attr_sim(): rsbac_get_attr() returned error!\n");
                      return(-RSBAC_EREADFAILED);
                    }
                  /* Derive data_type for this ipc item from owner's */
                  /* system_role */
                  if (i_attr_val1.system_role == SR_security_officer)
                    {
                      i_attr_val2.data_type = DT_SI;
                      /* set data_type for new IPC item */
                      if (rsbac_set_attr(T_IPC,
                                         tid,
                                         A_data_type,
                                         i_attr_val2))
                        {
                          printk(KERN_WARNING "rsbac_adf_set_attr_sim(): rsbac_set_attr() returned error!\n");
                          return(-RSBAC_EWRITEFAILED);
                        }
                    }
                  return(0);
                  
                /* all other cases are undefined */
                default:
                  return(0);
              }

/*********************/
        default: return(0);
      }

    return(0);
  }; /* end of rsbac_adf_set_attr_sim() */

/* end of rsbac/adf/sim/main.c */
